package org.sunbird.job.publish.helpers.spec

import akka.dispatch.ExecutionContexts
import com.typesafe.config.{Config, ConfigFactory}
import org.scalatest.{BeforeAndAfterAll, FlatSpec, Matchers}
import org.scalatestplus.mockito.MockitoSugar
import org.sunbird.job.content.publish.helpers.ExtractableMimeTypeHelper
import org.sunbird.job.content.task.ContentPublishConfig
import org.sunbird.job.exception.InvalidInputException
import org.sunbird.job.publish.core.ObjectData
import org.sunbird.job.util.CloudStorageUtil

class ExtractableMimeTypeHelperSpec extends FlatSpec with BeforeAndAfterAll with Matchers with MockitoSugar {

  implicit val ec = ExecutionContexts.global
  val config: Config = ConfigFactory.load("test.conf").withFallback(ConfigFactory.systemEnvironment())
  val jobConfig: ContentPublishConfig = new ContentPublishConfig(config)
  implicit val cloudStorageUtil = new CloudStorageUtil(jobConfig)

  "processECMLBody with xml " should " throw exception Error! Invalid Media ('id' is required.) in ... if media id is blank and type is other than js and css" in {
    val obj: ObjectData = new ObjectData("do_113188615625731",
      Map[String, AnyRef]("identifier" -> "do_113188615625731", "objectType" -> "Content", "mimeType" -> "application/vnd.ekstep.ecml-archive", "primaryCategory" -> "some category", "name" -> "Some Content", "code" -> "some code"),
      Some(Map[String, AnyRef]("body" -> "<theme><manifest><media plugin=\"org.ekstep.navigation\" id=\"\" ver=\"1.0\" src=\"content-plugins/org.ekstep.navigation-1.0/renderer/plugin.js\" type=\"plugin\"></media></manifest></theme>"))
    )

    assertThrows[InvalidInputException] {
      ExtractableMimeTypeHelper.processECMLBody(obj, jobConfig)(ec, cloudStorageUtil)
    }
  }

  "processECMLBody with xml " should " throw exception Error! Invalid Media ('type' is required.) in ... if media type is blank" in {
    val obj: ObjectData = new ObjectData("do_113188615625731",
      Map[String, AnyRef]("identifier" -> "do_113188615625731", "objectType" -> "Content", "mimeType" -> "application/vnd.ekstep.ecml-archive", "primaryCategory" -> "some category", "name" -> "Some Content", "code" -> "some code"),
      Some(Map[String, AnyRef]("body" -> "<theme><manifest><media plugin=\"org.ekstep.navigation\" id=\"org.ekstep.navigation\" ver=\"1.0\" src=\"content-plugins/org.ekstep.navigation-1.0/renderer/plugin.js\" type=\"\"></media></manifest></theme>"))
    )

    assertThrows[InvalidInputException] {
      ExtractableMimeTypeHelper.processECMLBody(obj, jobConfig)(ec, cloudStorageUtil)
    }
  }

  "processECMLBody with xml " should " throw exception Error! Invalid Media ('src' is required.) in ... if media src is blank" in {
    val obj: ObjectData = new ObjectData("do_113188615625731",
      Map[String, AnyRef]("identifier" -> "do_113188615625731", "objectType" -> "Content", "mimeType" -> "application/vnd.ekstep.ecml-archive", "primaryCategory" -> "some category", "name" -> "Some Content", "code" -> "some code"),
      Some(Map[String, AnyRef]("body" -> "<theme><manifest><media plugin=\"org.ekstep.navigation\" id=\"org.ekstep.navigation\" ver=\"1.0\" src=\"\" type=\"plugin\"></media></manifest></theme>"))
    )

    assertThrows[InvalidInputException] {
      ExtractableMimeTypeHelper.processECMLBody(obj, jobConfig)(ec, cloudStorageUtil)
    }
  }

  "processECMLBody with xml " should " process the ecml body" in {
    val obj: ObjectData = new ObjectData("do_113188615625731",
      Map[String, AnyRef]("identifier" -> "do_113188615625731", "objectType" -> "Content", "mimeType" -> "application/vnd.ekstep.ecml-archive", "primaryCategory" -> "some category", "name" -> "Some Content", "code" -> "some code"),
      Some(Map[String, AnyRef]("body" -> "<theme startStage=\"1dafad29-7454-fb86-6a6f-e5262d127a1c\" compatibilityVersion=\"2\" version=\"1.0\" id=\"theme\"><manifest><media plugin=\"org.ekstep.navigation\" id=\"org.ekstep.navigation_js\" ver=\"1.0\" src=\"content-plugins/org.ekstep.navigation-1.0/renderer/controller/navigation_ctrl.js\" type=\"js\"></media><media plugin=\"org.ekstep.navigation\" id=\"org.ekstep.navigation_html\" ver=\"1.0\" src=\"content-plugins/org.ekstep.navigation-1.0/renderer/templates/navigation.html\" type=\"js\"></media><media plugin=\"org.ekstep.navigation\" id=\"org.ekstep.navigation\" ver=\"1.0\" src=\"content-plugins/org.ekstep.navigation-1.0/renderer/plugin.js\" type=\"plugin\"></media><media plugin=\"org.ekstep.navigation\" id=\"org.ekstep.navigation_manifest\" ver=\"1.0\" src=\"content-plugins/org.ekstep.navigation-1.0/manifest.json\" type=\"json\"></media><media plugin=\"org.ekstep.iterator\" id=\"org.ekstep.iterator\" ver=\"1.0\" src=\"content-plugins/org.ekstep.iterator-1.0/renderer/plugin.js\" type=\"plugin\"></media><media plugin=\"org.ekstep.iterator\" id=\"org.ekstep.iterator_manifest\" ver=\"1.0\" src=\"content-plugins/org.ekstep.iterator-1.0/manifest.json\" type=\"json\"></media><media plugin=\"org.ekstep.questionset\" id=\"org.ekstep.questionset_telemetry_logger_js\" ver=\"1.0\" src=\"content-plugins/org.ekstep.questionset-1.0/renderer/utils/telemetry_logger.js\" type=\"js\"></media><media plugin=\"org.ekstep.questionset\" id=\"org.ekstep.questionset_audio_plugin_js\" ver=\"1.0\" src=\"content-plugins/org.ekstep.questionset-1.0/renderer/utils/html_audio_plugin.js\" type=\"js\"></media><media plugin=\"org.ekstep.questionset\" id=\"org.ekstep.questionset_feedback_popup_js\" ver=\"1.0\" src=\"content-plugins/org.ekstep.questionset-1.0/renderer/utils/qs_feedback_popup.js\" type=\"js\"></media><media plugin=\"org.ekstep.questionset\" id=\"org.ekstep.questionset\" ver=\"1.0\" src=\"content-plugins/org.ekstep.questionset-1.0/renderer/plugin.js\" type=\"plugin\"></media><media plugin=\"org.ekstep.questionset\" id=\"org.ekstep.questionset_manifest\" ver=\"1.0\" src=\"content-plugins/org.ekstep.questionset-1.0/manifest.json\" type=\"json\"></media><media plugin=\"org.ekstep.questionunit\" id=\"org.ekstep.questionunit.renderer.audioicon\" ver=\"1.1\" src=\"content-plugins/org.ekstep.questionunit-1.1/renderer/assets/audio-icon.png\" type=\"image\"></media><media plugin=\"org.ekstep.questionunit\" id=\"org.ekstep.questionunit.renderer.downarrow\" ver=\"1.1\" src=\"content-plugins/org.ekstep.questionunit-1.1/renderer/assets/down_arrow.png\" type=\"image\"></media><media plugin=\"org.ekstep.questionunit\" id=\"org.ekstep.questionunit_js\" ver=\"1.1\" src=\"content-plugins/org.ekstep.questionunit-1.1/renderer/components/js/components.js\" type=\"js\"></media><media plugin=\"org.ekstep.questionunit\" id=\"org.ekstep.questionunit_css\" ver=\"1.1\" src=\"content-plugins/org.ekstep.questionunit-1.1/renderer/components/css/components.css\" type=\"css\"></media><media plugin=\"org.ekstep.questionunit\" id=\"org.ekstep.questionunit\" ver=\"1.1\" src=\"content-plugins/org.ekstep.questionunit-1.1/renderer/plugin.js\" type=\"plugin\"></media><media plugin=\"org.ekstep.questionunit\" id=\"org.ekstep.questionunit_manifest\" ver=\"1.1\" src=\"content-plugins/org.ekstep.questionunit-1.1/manifest.json\" type=\"json\"></media><media plugin=\"org.sunbird.questionunit.quml\" id=\"org.sunbird.questionunit.quml_manifest\" ver=\"1.1\" src=\"content-plugins/org.sunbird.questionunit.quml-1.1/manifest.json\" type=\"json\"></media><media plugin=\"org.sunbird.questionunit.quml\" id=\"org.sunbird.questionunit.quml.plugin_js\" ver=\"1.1\" src=\"content-plugins/org.sunbird.questionunit.quml-1.1/renderer/plugin.js\" type=\"plugin\"></media><media plugin=\"org.sunbird.questionunit.quml\" id=\"org.sunbird.questionunit.quml.feedback_popup\" ver=\"1.1\" src=\"content-plugins/org.sunbird.questionunit.quml-1.1/renderer/utils/quml_feedback_popup.js\" type=\"js\"></media><media plugin=\"org.sunbird.questionunit.quml\" id=\"org.sunbird.questionunit.quml.feedback_close\" ver=\"1.1\" src=\"content-plugins/org.sunbird.questionunit.quml-1.1/renderer/assets/feedback-close.svg\" type=\"image\"></media><media plugin=\"org.sunbird.questionunit.quml\" id=\"org.sunbird.questionunit.quml.play\" ver=\"1.1\" src=\"content-plugins/org.sunbird.questionunit.quml-1.1/renderer/assets/player-play-button.png\" type=\"image\"></media><media plugin=\"org.sunbird.questionunit.quml\" id=\"org.sunbird.questionunit.quml_css\" ver=\"1.1\" src=\"content-plugins/org.sunbird.questionunit.quml-1.1/renderer/styles/style.css\" type=\"css\"></media><media plugin=\"org.ekstep.questionunit\" id=\"4b2b66a2-9c5b-42d5-891d-ec0c3d1d516c\" ver=\"1.1\" src=\"/content-plugins/org.ekstep.questionunit-1.1/renderer/libs/katex/katex.min.js\" type=\"js\"></media><media plugin=\"org.ekstep.questionunit\" id=\"7e02c56a-c048-444d-bb2a-d4b854723adc\" ver=\"1.1\" src=\"content-plugins/org.ekstep.questionunit-1.1/renderer/libs/katex/katex.min.css\" type=\"css\"></media><media plugin=\"org.ekstep.questionunit\" id=\"04ad980a-88ac-4c55-a597-266e42240670\" ver=\"1.1\" src=\"content-plugins/org.ekstep.questionunit-1.1/renderer/libs/katex/fonts/katex_main-bold.ttf\" type=\"js\"></media><media plugin=\"org.ekstep.questionunit\" id=\"75ab3bcd-2c52-47b0-8b46-b635256ce06a\" ver=\"1.1\" src=\"content-plugins/org.ekstep.questionunit-1.1/renderer/libs/katex/fonts/katex_main-bolditalic.ttf\" type=\"js\"></media><media plugin=\"org.ekstep.questionunit\" id=\"098cc69e-8658-483e-8597-8c392a1b3545\" ver=\"1.1\" src=\"content-plugins/org.ekstep.questionunit-1.1/renderer/libs/katex/fonts/katex_main-italic.ttf\" type=\"js\"></media><media plugin=\"org.ekstep.questionunit\" id=\"102d1165-9544-480b-9ec1-1cb83937e650\" ver=\"1.1\" src=\"content-plugins/org.ekstep.questionunit-1.1/renderer/libs/katex/fonts/katex_main-regular.ttf\" type=\"js\"></media><media plugin=\"org.ekstep.questionunit\" id=\"3bc5dce9-51ca-4a59-a0d9-d87d47c44670\" ver=\"1.1\" src=\"content-plugins/org.ekstep.questionunit-1.1/renderer/libs/katex/fonts/katex_math-bolditalic.ttf\" type=\"js\"></media><media plugin=\"org.ekstep.questionunit\" id=\"a95bb693-d688-4e7a-a9bc-08e0cb6d1d62\" ver=\"1.1\" src=\"content-plugins/org.ekstep.questionunit-1.1/renderer/libs/katex/fonts/katex_math-italic.ttf\" type=\"js\"></media><media plugin=\"org.ekstep.questionunit\" id=\"15757105-9023-4773-ba2a-8e619a0dc3a6\" ver=\"1.1\" src=\"content-plugins/org.ekstep.questionunit-1.1/renderer/libs/katex/fonts/katex_math-regular.ttf\" type=\"js\"></media><media plugin=\"org.ekstep.questionunit\" id=\"5b1a9e58-8b7e-4295-a90d-593dc831262a\" ver=\"1.1\" src=\"content-plugins/org.ekstep.questionunit-1.1/renderer/libs/katex/fonts/katex_size1-regular.ttf\" type=\"js\"></media><media plugin=\"org.ekstep.questionunit\" id=\"bc6ee547-9db0-480e-b298-b5509d43207a\" ver=\"1.1\" src=\"content-plugins/org.ekstep.questionunit-1.1/renderer/libs/katex/fonts/katex_size2-regular.ttf\" type=\"js\"></media><media plugin=\"org.ekstep.questionunit\" id=\"35b0245e-d7d3-4ac0-9cac-6c2c328c8948\" ver=\"1.1\" src=\"content-plugins/org.ekstep.questionunit-1.1/renderer/libs/katex/fonts/katex_size3-regular.ttf\" type=\"js\"></media><media plugin=\"org.ekstep.questionunit\" id=\"80a19118-48eb-4859-9b0c-576a90a01148\" ver=\"1.1\" src=\"content-plugins/org.ekstep.questionunit-1.1/renderer/libs/katex/fonts/katex_size4-regular.ttf\" type=\"js\"></media></manifest><stage x=\"0\" y=\"0\" id=\"1dafad29-7454-fb86-6a6f-e5262d127a1c\" h=\"100\" w=\"100\"><config><![CDATA[{'opacity':100,'strokeWidth':1,'stroke':'rgba(255, 255, 255, 0)','autoplay':false,'visible':true,'color':'#FFFFFF','genieControls':false,'instructions':''}]]></config><org.ekstep.questionset z-index=\"0\" x=\"9\" y=\"6\" rotate=\"0\" id=\"69672a7f-a428-a510-607f-e5f14c050915\" h=\"85\" w=\"80\"><data><![CDATA[[{\"identifier\":\"69672a7f-a428-a510-607f-e5f14c050915\"}]]]></data><config><![CDATA[{\"max_score\":1,\"allow_skip\":true,\"show_feedback\":true,\"shuffle_questions\":false,\"shuffle_options\":false,\"total_items\":1}]]></config><org.ekstep.question x=\"9\" y=\"6\" pluginId=\"org.sunbird.questionunit.quml\" id=\"54acb83e-e6a7-32c6-5cb6-29d16586ccc6\" templateId=\"qumltemplate\" h=\"85\" type=\"quml\" w=\"80\" pluginVer=\"1.1\"><data><![CDATA[{\"question\":\"<div class='mcq-horizontal cheveron-helper'><div class='mcq-title'><p>A person having less ‘haemoglobin’ is suffering from:</p></div><i class='chevron down icon'></i><div class='mcq-options'><div data-simple-choice-interaction data-response-variable='responseValue' value=0 class='mcq-option'><p>Jaundice</p></div><div data-simple-choice-interaction data-response-variable='responseValue' value=1 class='mcq-option'><p>Anaemia</p></div><div data-simple-choice-interaction data-response-variable='responseValue' value=2 class='mcq-option'><p>Malaria</p></div><div data-simple-choice-interaction data-response-variable='responseValue' value=3 class='mcq-option'><p>Chikungunya</p></div></div></div>\",\"media\":[],\"responseDeclaration\":{\"responseValue\":{\"cardinality\":\"single\",\"type\":\"integer\",\"correct_response\":{\"value\":\"2\"}}},\"options\":[{\"answer\":false,\"value\":{\"type\":\"text\",\"body\":\"<p>Jaundice</p>\",\"resvalue\":0,\"resindex\":0}},{\"answer\":false,\"value\":{\"type\":\"text\",\"body\":\"<p>Anaemia</p>\",\"resvalue\":1,\"resindex\":1}},{\"answer\":true,\"value\":{\"type\":\"text\",\"body\":\"<p>Malaria</p>\",\"resvalue\":2,\"resindex\":2}},{\"answer\":false,\"value\":{\"type\":\"text\",\"body\":\"<p>Chikungunya</p>\",\"resvalue\":3,\"resindex\":3}}],\"questionCount\":0}]]></data><config><![CDATA[{\"max_score\":1,\"partial_scoring\":false,\"isShuffleOption\":false,\"responseDeclaration\":{},\"metadata\":{\"itemType\":\"UNIT\",\"code\":\"e8c45c21-82ac-de15-1885-5ae030639a2c\",\"subject\":\"Environmental Studies\",\"qlevel\":\"MEDIUM\",\"qumlVersion\":1,\"channel\":\"01309282781705830427\",\"responseDeclaration\":{\"responseValue\":{\"cardinality\":\"single\",\"type\":\"integer\",\"correct_response\":{\"value\":\"2\"}}},\"language\":[\"English\"],\"medium\":\"English\",\"type\":\"mcq\",\"templateId\":\"mcq-horizontal\",\"createdOn\":\"2021-02-10T08:39:21.748+0000\",\"gradeLevel\":[\"Class 5\"],\"appId\":\"dev.dock.portal\",\"lastUpdatedOn\":\"2021-02-10T08:41:04.338+0000\",\"identifier\":\"do_1132132565954396161724\",\"creator\":\"N18\",\"lastStatusChangedOn\":\"2021-02-10T08:39:21.748+0000\",\"author\":\"N18\",\"maxScore\":1,\"version\":3,\"versionKey\":\"1612946464338\",\"framework\":\"ekstep_ncert_k-12\",\"rejectComment\":\"\",\"name\":\"mcq_ekstep_ncert_k-12\",\"category\":\"MCQ\",\"board\":\"CBSE\",\"programId\":\"0c0246b0-6b7a-11eb-aec2-119aec14cb6e\",\"status\":\"Draft\"}}]]></config></org.ekstep.question></org.ekstep.questionset></stage><plugin-manifest><plugin depends=\"\" id=\"org.ekstep.navigation\" ver=\"1.0\" type=\"plugin\"></plugin><plugin depends=\"\" id=\"org.ekstep.questionunit\" ver=\"1.1\" type=\"plugin\"></plugin><plugin depends=\"\" id=\"org.ekstep.iterator\" ver=\"1.0\" type=\"plugin\"></plugin><plugin depends=\"org.ekstep.questionset.quiz,org.ekstep.iterator\" id=\"org.ekstep.questionset\" ver=\"1.0\" type=\"plugin\"></plugin><plugin depends=\"org.ekstep.questionunit\" id=\"org.sunbird.questionunit.quml\" ver=\"1.1\" type=\"plugin\"></plugin></plugin-manifest></theme>"))
    )
    val result: Map[String, AnyRef] = ExtractableMimeTypeHelper.processECMLBody(obj, jobConfig)(ec, cloudStorageUtil)
    result.contains("artifactUrl") shouldBe true
    result.get("artifactUrl") shouldNot be(null)
    result.contains("cloudStorageKey") shouldBe true
    result.get("cloudStorageKey") shouldNot be(null)
  }

  "processECMLBody with json " should " throw exception if no body is available" in {
    val obj: ObjectData = new ObjectData("do_113188615625731",
      Map[String, AnyRef]("identifier" -> "do_113188615625731", "objectType" -> "Content", "mimeType" -> "application/vnd.ekstep.ecml-archive", "primaryCategory" -> "some category", "name" -> "Some Content", "code" -> "some code"),
      Some(Map[String, AnyRef]())
    )

    assertThrows[InvalidInputException] {
      val result: Map[String, AnyRef] = ExtractableMimeTypeHelper.processECMLBody(obj, jobConfig)(ec, cloudStorageUtil)
    }
  }

  "processECMLBody with json " should " throw exception Invalid Content Body if not a valid body" in {
    val obj: ObjectData = new ObjectData("do_113188615625731",
      Map[String, AnyRef]("identifier" -> "do_113188615625731", "objectType" -> "Content", "mimeType" -> "application/vnd.ekstep.ecml-archive", "primaryCategory" -> "some category", "name" -> "Some Content", "code" -> "some code"),
      Some(Map[String, AnyRef]("body" -> "Invalid body"))
    )

    assertThrows[InvalidInputException] {
      ExtractableMimeTypeHelper.processECMLBody(obj, jobConfig)(ec, cloudStorageUtil)
    }
  }

  "processECMLBody with json " should " throw exception Error! Invalid Controller ('id' is required.)" in {
    val obj: ObjectData = new ObjectData("do_113188615625731",
      Map[String, AnyRef]("identifier" -> "do_113188615625731", "objectType" -> "Content", "mimeType" -> "application/vnd.ekstep.ecml-archive", "primaryCategory" -> "some category", "name" -> "Some Content", "code" -> "some code"),
      Some(Map[String, AnyRef]("body" -> "{\"theme\":{\"controller\":[{\"name\":\"dictionary\",\"type\":\"data\",\"id\":\"\",\"__cdata\":{}}]}}"))
    )

    assertThrows[InvalidInputException] {
      ExtractableMimeTypeHelper.processECMLBody(obj, jobConfig)(ec, cloudStorageUtil)
    }
  }

  "processECMLBody with json " should " throw exception Error! Invalid Controller ('type' is required.)" in {
    val obj: ObjectData = new ObjectData("do_113188615625731",
      Map[String, AnyRef]("identifier" -> "do_113188615625731", "objectType" -> "Content", "mimeType" -> "application/vnd.ekstep.ecml-archive", "primaryCategory" -> "some category", "name" -> "Some Content", "code" -> "some code"),
      Some(Map[String, AnyRef]("body" -> "{\"theme\":{\"controller\":[{\"name\":\"dictionary\",\"type\":\"\",\"id\":\"dictionary\",\"__cdata\":{}}]}}"))
    )

    assertThrows[InvalidInputException] {
      ExtractableMimeTypeHelper.processECMLBody(obj, jobConfig)(ec, cloudStorageUtil)
    }
  }

  "processECMLBody with json " should " throw exception Error! Invalid Controller ('type' should be either 'items' or 'data')" in {
    val obj: ObjectData = new ObjectData("do_113188615625731",
      Map[String, AnyRef]("identifier" -> "do_113188615625731", "objectType" -> "Content", "mimeType" -> "application/vnd.ekstep.ecml-archive", "primaryCategory" -> "some category", "name" -> "Some Content", "code" -> "some code"),
      Some(Map[String, AnyRef]("body" -> "{\"theme\":{\"controller\":[{\"name\":\"dictionary\",\"type\":\"some type\",\"id\":\"dictionary\",\"__cdata\":{}}]}}"))
    )

    assertThrows[InvalidInputException] {
      ExtractableMimeTypeHelper.processECMLBody(obj, jobConfig)(ec, cloudStorageUtil)
    }
  }

  "processECMLBody with json " should " throw exception Error! Invalid Media ('id' is required.) if media id is blank and type is other than js and css" in {
    val obj: ObjectData = new ObjectData("do_113188615625731",
      Map[String, AnyRef]("identifier" -> "do_113188615625731", "objectType" -> "Content", "mimeType" -> "application/vnd.ekstep.ecml-archive", "primaryCategory" -> "some category", "name" -> "Some Content", "code" -> "some code"),
      Some(Map[String, AnyRef]("body" -> "{\"theme\":{\"controller\":[{\"name\":\"dictionary\",\"type\":\"data\",\"id\":\"dictionary\",\"__cdata\":{}}],\"manifest\":{\"media\":[{\"id\":\"\",\"src\":\"/content-plugins/org.ekstep.questionset-1.0/editor/assets/quizimage.png\",\"assetId\":\"QuizImage\",\"type\":\"image\",\"preload\":true}]}}}"))
    )

    assertThrows[InvalidInputException] {
      ExtractableMimeTypeHelper.processECMLBody(obj, jobConfig)(ec, cloudStorageUtil)
    }
  }

  "processECMLBody with json " should " throw exception Error! Invalid Media ('type' is required.) if media type is blank" in {
    val obj: ObjectData = new ObjectData("do_113188615625731",
      Map[String, AnyRef]("identifier" -> "do_113188615625731", "objectType" -> "Content", "mimeType" -> "application/vnd.ekstep.ecml-archive", "primaryCategory" -> "some category", "name" -> "Some Content", "code" -> "some code"),
      Some(Map[String, AnyRef]("body" -> "{\"theme\":{\"controller\":[{\"name\":\"dictionary\",\"type\":\"data\",\"id\":\"dictionary\",\"__cdata\":{}}],\"manifest\":{\"media\":[{\"id\":\"QuizImage\",\"src\":\"/content-plugins/org.ekstep.questionset-1.0/editor/assets/quizimage.png\",\"assetId\":\"QuizImage\",\"type\":\"\",\"preload\":true}]}}}"))
    )

    assertThrows[InvalidInputException] {
      ExtractableMimeTypeHelper.processECMLBody(obj, jobConfig)(ec, cloudStorageUtil)
    }
  }

  "processECMLBody with json " should " throw exception Error! Invalid Media ('src' is required.) if media src is blank" in {
    val obj: ObjectData = new ObjectData("do_113188615625731",
      Map[String, AnyRef]("identifier" -> "do_113188615625731", "objectType" -> "Content", "mimeType" -> "application/vnd.ekstep.ecml-archive", "primaryCategory" -> "some category", "name" -> "Some Content", "code" -> "some code"),
      Some(Map[String, AnyRef]("body" -> "{\"theme\":{\"controller\":[{\"name\":\"dictionary\",\"type\":\"data\",\"id\":\"dictionary\",\"__cdata\":{}}],\"manifest\":{\"media\":[{\"id\":\"QuizImage\",\"src\":\"\",\"assetId\":\"QuizImage\",\"type\":\"image\",\"preload\":true}]}}}"))
    )

    assertThrows[InvalidInputException] {
      ExtractableMimeTypeHelper.processECMLBody(obj, jobConfig)(ec, cloudStorageUtil)
    }
  }

  "processECMLBody with json " should " process the ecml body" in {
    val obj: ObjectData = new ObjectData("do_113188615625731",
      Map[String, AnyRef]("identifier" -> "do_113188615625731", "objectType" -> "Content", "mimeType" -> "application/vnd.ekstep.ecml-archive", "primaryCategory" -> "some category", "name" -> "Some Content", "code" -> "some code"),
      Some(Map[String, AnyRef]("body" -> "{\"theme\":{\"manifest\":{\"media\":[{\"id\":253452185,\"plugin\":\"org.ekstep.navigation\",\"ver\":\"1.0\",\"src\":\"/content-plugins/org.ekstep.navigation-1.0/renderer/controller/navigation_ctrl.js\",\"type\":\"js\"},{\"id\":\"c264cc74-39dc-40e6-8e6a-04b1b379bd52\",\"plugin\":\"org.ekstep.navigation\",\"ver\":\"1.0\",\"src\":\"/content-plugins/org.ekstep.navigation-1.0/renderer/templates/navigation.html\",\"type\":\"js\"},{\"id\":\"org.ekstep.navigation\",\"plugin\":\"org.ekstep.navigation\",\"ver\":\"1.0\",\"src\":\"/content-plugins/org.ekstep.navigation-1.0/renderer/plugin.js\",\"type\":\"plugin\"},{\"id\":\"org.ekstep.navigation_manifest\",\"plugin\":\"org.ekstep.navigation\",\"ver\":\"1.0\",\"src\":\"/content-plugins/org.ekstep.navigation-1.0/manifest.json\",\"type\":\"json\"},{\"id\":\"org.ekstep.questionunit.renderer.audioicon\",\"plugin\":\"org.ekstep.questionunit\",\"ver\":\"1.0\",\"src\":\"/content-plugins/org.ekstep.questionunit-1.0/renderer/assets/audio-icon.png\",\"type\":\"image\"},{\"id\":\"org.ekstep.questionunit.renderer.downarrow\",\"plugin\":\"org.ekstep.questionunit\",\"ver\":\"1.0\",\"src\":\"/content-plugins/org.ekstep.questionunit-1.0/renderer/assets/down_arrow.png\",\"type\":\"image\"},{\"id\":\"489ca96e-de6d-41c2-a346-3e13a5e81cd1\",\"plugin\":\"org.ekstep.questionunit\",\"ver\":\"1.0\",\"src\":\"/content-plugins/org.ekstep.questionunit-1.0/renderer/components/js/components.js\",\"type\":\"js\"},{\"id\":\"d3eab6b4-eb5a-4739-b985-ca85caaae11e\",\"plugin\":\"org.ekstep.questionunit\",\"ver\":\"1.0\",\"src\":\"/content-plugins/org.ekstep.questionunit-1.0/renderer/components/css/components.css\",\"type\":\"css\"},{\"id\":\"4152517e-7698-4813-840c-ab0d4bb1a48a\",\"plugin\":\"org.ekstep.questionunit\",\"ver\":\"1.0\",\"src\":\"/content-plugins/org.ekstep.questionunit-1.0/renderer/libs/katex/katex.min.js\",\"type\":\"js\"},{\"id\":\"d9a519d7-e450-4b31-be54-5cec9d21d5ce\",\"plugin\":\"org.ekstep.questionunit\",\"ver\":\"1.0\",\"src\":\"/content-plugins/org.ekstep.questionunit-1.0/renderer/libs/katex/katex.min.css\",\"type\":\"css\"},{\"id\":\"org.ekstep.questionunit\",\"plugin\":\"org.ekstep.questionunit\",\"ver\":\"1.0\",\"src\":\"/content-plugins/org.ekstep.questionunit-1.0/renderer/plugin.js\",\"type\":\"plugin\"},{\"id\":\"org.ekstep.questionunit_manifest\",\"plugin\":\"org.ekstep.questionunit\",\"ver\":\"1.0\",\"src\":\"/content-plugins/org.ekstep.questionunit-1.0/manifest.json\",\"type\":\"json\"},{\"id\":\"7139ec8e-dc5d-437b-a7c4-62d0be5a28c5\",\"plugin\":\"org.ekstep.questionunit.mcq\",\"ver\":\"1.1\",\"src\":\"/content-plugins/org.ekstep.questionunit.mcq-1.1/renderer/styles/style.css\",\"type\":\"css\"},{\"id\":\"bdd3831b-95ca-4398-a6e6-6a61984452a5\",\"plugin\":\"org.ekstep.questionunit.mcq\",\"ver\":\"1.1\",\"src\":\"/content-plugins/org.ekstep.questionunit.mcq-1.1/renderer/styles/horizontal_and_vertical.css\",\"type\":\"css\"},{\"id\":\"09bd898a-6132-4ebd-936c-5437a2134229\",\"plugin\":\"org.ekstep.questionunit.mcq\",\"ver\":\"1.1\",\"src\":\"/content-plugins/org.ekstep.questionunit.mcq-1.1/renderer/template/mcq-layouts.js\",\"type\":\"js\"},{\"id\":\"a46352ba-dccb-4c5e-bc1a-84beeb85037c\",\"plugin\":\"org.ekstep.questionunit.mcq\",\"ver\":\"1.1\",\"src\":\"/content-plugins/org.ekstep.questionunit.mcq-1.1/renderer/template/template_controller.js\",\"type\":\"js\"},{\"id\":\"38886295-0bac-4654-bded-6d032cb37431\",\"plugin\":\"org.ekstep.questionunit.mcq\",\"ver\":\"1.1\",\"src\":\"/content-plugins/org.ekstep.questionunit.mcq-1.1/renderer/assets/tick_icon.png\",\"type\":\"image\"},{\"id\":\"37aabee2-1538-4391-8d1d-5d406722b084\",\"plugin\":\"org.ekstep.questionunit.mcq\",\"ver\":\"1.1\",\"src\":\"/content-plugins/org.ekstep.questionunit.mcq-1.1/renderer/assets/audio-icon2.png\",\"type\":\"image\"},{\"id\":\"73d9743a-9202-4e68-8797-ad873b67cdb6\",\"plugin\":\"org.ekstep.questionunit.mcq\",\"ver\":\"1.1\",\"src\":\"/content-plugins/org.ekstep.questionunit.mcq-1.1/renderer/assets/music-blue.png\",\"type\":\"image\"},{\"id\":\"org.ekstep.questionunit.mcq\",\"plugin\":\"org.ekstep.questionunit.mcq\",\"ver\":\"1.1\",\"src\":\"/content-plugins/org.ekstep.questionunit.mcq-1.1/renderer/plugin.js\",\"type\":\"plugin\"},{\"id\":\"org.ekstep.questionunit.mcq_manifest\",\"plugin\":\"org.ekstep.questionunit.mcq\",\"ver\":\"1.1\",\"src\":\"/content-plugins/org.ekstep.questionunit.mcq-1.1/manifest.json\",\"type\":\"json\"},{\"id\":\"org.ekstep.questionset.quiz\",\"plugin\":\"org.ekstep.questionset.quiz\",\"ver\":\"1.0\",\"src\":\"/content-plugins/org.ekstep.questionset.quiz-1.0/renderer/plugin.js\",\"type\":\"plugin\"},{\"id\":\"org.ekstep.questionset.quiz_manifest\",\"plugin\":\"org.ekstep.questionset.quiz\",\"ver\":\"1.0\",\"src\":\"/content-plugins/org.ekstep.questionset.quiz-1.0/manifest.json\",\"type\":\"json\"},{\"id\":\"org.ekstep.iterator\",\"plugin\":\"org.ekstep.iterator\",\"ver\":\"1.0\",\"src\":\"/content-plugins/org.ekstep.iterator-1.0/renderer/plugin.js\",\"type\":\"plugin\"},{\"id\":\"org.ekstep.iterator_manifest\",\"plugin\":\"org.ekstep.iterator\",\"ver\":\"1.0\",\"src\":\"/content-plugins/org.ekstep.iterator-1.0/manifest.json\",\"type\":\"json\"},{\"id\":\"cb4d4669-9291-4583-906c-685ab9a76994\",\"plugin\":\"org.ekstep.questionset\",\"ver\":\"1.0\",\"src\":\"/content-plugins/org.ekstep.questionset-1.0/renderer/utils/telemetry_logger.js\",\"type\":\"js\"},{\"id\":\"dd12429d-d861-4d54-b80d-12047823b058\",\"plugin\":\"org.ekstep.questionset\",\"ver\":\"1.0\",\"src\":\"/content-plugins/org.ekstep.questionset-1.0/renderer/utils/html_audio_plugin.js\",\"type\":\"js\"},{\"id\":\"56254ade-8e4e-4090-b97b-8b9b7bc12620\",\"plugin\":\"org.ekstep.questionset\",\"ver\":\"1.0\",\"src\":\"/content-plugins/org.ekstep.questionset-1.0/renderer/utils/qs_feedback_popup.js\",\"type\":\"js\"},{\"id\":\"org.ekstep.questionset\",\"plugin\":\"org.ekstep.questionset\",\"ver\":\"1.0\",\"src\":\"/content-plugins/org.ekstep.questionset-1.0/renderer/plugin.js\",\"type\":\"plugin\"},{\"id\":\"org.ekstep.questionset_manifest\",\"plugin\":\"org.ekstep.questionset\",\"ver\":\"1.0\",\"src\":\"/content-plugins/org.ekstep.questionset-1.0/manifest.json\",\"type\":\"json\"},{\"id\":\"QuizImage\",\"src\":\"/content-plugins/org.ekstep.questionset-1.0/editor/assets/quizimage.png\",\"assetId\":\"QuizImage\",\"type\":\"image\",\"preload\":true}]},\"id\":\"theme\",\"startStage\":\"scene55138843-9f04-487d-bf33-1920769f2c96\",\"ver\":0.2,\"controller\":[{\"name\":\"assessment_185d692a-1841-4a35-96c1-67cebcde42a2\",\"type\":\"items\",\"id\":\"assessment_185d692a-1841-4a35-96c1-67cebcde42a2\",\"__cdata\":{\"total_items\":1,\"subject\":\"domain\",\"code\":\"ItemSet_921ee047-c676-4e0b-9a71-42f371f7b843\",\"type\":\"materialised\",\"lastUpdatedOn\":\"2016-06-15T09:42:19.585+0000\",\"showImmediateFeedback\":true,\"SET_TYPE\":\"MATERIALISED_SET\",\"createdOn\":\"2016-06-15T09:42:19.575+0000\",\"title\":\"Assessment Title\",\"items\":{\"domain_11824\":[{\"model\":{\"keys\":\"र,रा,रि,री,रु,रू,रे,रै,रो,रौ,रं,रः,ट,टा,टि,टी,टु,टू,टे,टै,टो,टौ,टं,टः\"},\"question_audio\":\"domain_10561\",\"subject\":\"domain\",\"template_id\":\"domain_4171\",\"type\":\"ftb\",\"feedback\":\"\",\"qlevel\":\"EASY\",\"title\":\"Story5Q1\",\"question_image\":\"domain_10624\",\"name\":\"Story5Q1\",\"domain\":\"literacy\",\"max_score\":1,\"question\":\"\",\"template\":\"org.ekstep.ftb.barakhadi\",\"answer\":{\"ans1\":{\"value\":\"रोटी\",\"score\":1}},\"code\":\"org.ekstep.assessmentitem.domain_4533\",\"lastUpdatedOn\":\"2016-06-15T09:40:54.289+0000\",\"concepts\":[{\"identifier\":\"LO1\",\"name\":\"Receptive Vocabulary\",\"objectType\":\"Concept\",\"relation\":\"associatedTo\",\"description\":null,\"index\":null}],\"createdOn\":\"2016-05-23T10:09:03.976+0000\",\"lastUpdatedBy\":\"323\",\"used_for\":\"worksheet\",\"owner\":\"323\",\"gradeLevel\":[\"Other\"],\"language\":[\"Hindi\"],\"identifier\":\"domain_4533\",\"media\":[{\"id\":\"domain_10624\",\"type\":\"image\",\"src\":\"https://ekstep-public.s3-ap-southeast-1.amazonaws.com/content/roti_323_1465971255_1465971255545.jpg\",\"asset_id\":\"domain_10624\",\"preload\":true},{\"id\":\"domain_10561\",\"type\":\"audio\",\"src\":\"https://ekstep-public.s3-ap-southeast-1.amazonaws.com/content/story11q4a1_141_1465970351_1465970351272.mp3\",\"asset_id\":\"domain_10561\",\"preload\":true}]}]},\"SET_OBJECT_TYPE_KEY\":\"AssessmentItem\",\"shuffle\":false,\"lastUpdatedBy\":\"323\",\"owner\":\"Ilimi\",\"used_for\":\"assessment\",\"max_score\":1,\"gradeLevel\":[\"Grade 1\"],\"item_sets\":[{\"id\":\"domain_11824\",\"count\":1}],\"language\":[\"English\"],\"identifier\":\"domain_11824\"}},{\"name\":\"dictionary\",\"type\":\"data\",\"id\":\"dictionary\",\"__cdata\":{}}],\"template\":[{\"text\":[{\"align\":\"center\",\"color\":\"black\",\"font\":\"Verdana\",\"fontsize\":70,\"model\":\"item.title\",\"w\":80,\"x\":10,\"y\":6,\"z-index\":101},{\"align\":\"center\",\"color\":\"black\",\"font\":\"Verdana\",\"fontsize\":100,\"h\":15,\"id\":\"newText\",\"model\":\"item.ans1\",\"valign\":\"middle\",\"w\":35,\"x\":58,\"y\":68,\"z-index\":100}],\"shape\":[{\"event\":{\"type\":\"click\"},\"h\":15,\"hitArea\":true,\"opacity\":1,\"w\":80,\"x\":10,\"y\":6,\"z-index\":99},{\"event\":{\"action\":{\"asset\":\"bKeyboard\",\"command\":\"custom\",\"id\":\"newText\",\"invoke\":\"switchTarget\",\"type\":\"command\"},\"type\":\"click\"},\"h\":15,\"hitArea\":true,\"stroke\":\"black\",\"stroke-width\":5,\"w\":35,\"x\":58,\"y\":67,\"z-index\":99},{\"event\":{\"action\":{\"asset_model\":\"item.question_audio\",\"command\":\"play\",\"type\":\"command\"},\"type\":\"click\"},\"h\":40,\"hitArea\":true,\"stroke-width\":5,\"w\":20,\"x\":10,\"y\":58,\"z-index\":99}],\"keyboard\":{\"id\":\"bKeyboard\",\"keys\":\"item.keys\",\"limit\":10,\"target\":\"newText\",\"type\":\"custom\",\"x\":5,\"y\":15},\"g\":{\"image\":{\"h\":75,\"model\":\"item.question_image\",\"w\":100,\"x\":0,\"y\":0,\"z-index\":102},\"text\":{\"align\":\"center\",\"color\":\"black\",\"font\":\"Verdana\",\"fontsize\":\"2em\",\"h\":25,\"model\":\"item.question\",\"valign\":\"middle\",\"w\":100,\"x\":0,\"y\":75,\"z-index\":103},\"h\":40,\"w\":20,\"x\":10,\"y\":58},\"id\":\"org.ekstep.ftb.barakhadi\"},{\"text\":{\"event\":{\"action\":[{\"command\":\"stop\",\"sound\":true,\"type\":\"command\"},{\"asset_model\":\"item.question_audio\",\"command\":\"play\",\"type\":\"command\"}],\"type\":\"click\"},\"align\":\"center\",\"color\":\"black\",\"font\":\"Verdana\",\"fontsize\":\"2em\",\"lineHeight\":2,\"model\":\"item.question\",\"valign\":\"middle\",\"w\":100,\"x\":0,\"y\":15},\"shape\":{\"event\":{\"action\":[{\"command\":\"stop\",\"sound\":true,\"type\":\"command\"},{\"asset_model\":\"item.question_audio\",\"command\":\"play\",\"type\":\"command\"}],\"type\":\"click\"},\"h\":10,\"hitArea\":true,\"type\":\"rect\",\"w\":100,\"x\":0,\"y\":15},\"mcq\":{\"options\":{\"shape\":{\"h\":100,\"stroke\":\"black\",\"type\":\"roundrect\",\"w\":100,\"x\":0,\"y\":0},\"text\":{\"align\":\"center\",\"font\":\"Verdana\",\"fontsize\":\"1.6em\",\"h\":100,\"model\":\"option.value.text\",\"valign\":\"middle\",\"w\":100,\"x\":0,\"y\":0},\"event\":{\"action\":[{\"command\":\"stop\",\"sound\":true,\"type\":\"command\"},{\"asset_model\":\"option.value.audio\",\"command\":\"play\",\"type\":\"command\"}],\"type\":\"click\"},\"cols\":4,\"h\":30,\"highlight\":\"yellow\",\"layout\":\"table\",\"marginX\":5,\"marginY\":5,\"options\":\"options\",\"w\":97,\"x\":1.5,\"y\":50},\"model\":\"item\",\"multi_select\":false,\"shadow\":\"#56B1F7\"},\"id\":\"org.ekstep.mcq.ta\"},{\"text\":{\"model\":\"item.title\",\"x\":9,\"y\":7,\"w\":86,\"h\":4,\"font\":\"Georgia\",\"fontsize\":42},\"mcq\":{\"options\":{\"layout\":\"table\",\"x\":20,\"y\":15,\"w\":70,\"h\":85,\"cols\":2,\"marginX\":10,\"marginY\":5,\"options\":\"options\"},\"multi_select\":false,\"model\":\"item\"},\"g\":{\"image\":{\"event\":{\"action\":[{\"type\":\"command\",\"command\":\"stop\",\"asset_model\":\"item.hints[0].asset\"},{\"type\":\"command\",\"command\":\"toggleShow\",\"asset\":\"hint\"}],\"type\":\"click\"},\"asset\":\"speech_bubble\",\"x\":0,\"y\":0,\"w\":100,\"h\":100},\"text\":[{\"x\":10,\"y\":20,\"w\":80,\"h\":80,\"font\":\"Georgia\",\"weight\":\"bold\",\"fontsize\":150,\"__text\":\"Hint\"},{\"x\":10,\"y\":40,\"w\":80,\"h\":80,\"font\":\"Georgia\",\"fontsize\":120,\"model\":\"item.hints[1].asset\"}],\"x\":9,\"y\":17,\"w\":20,\"h\":20,\"id\":\"hint\",\"visible\":false},\"image\":{\"event\":{\"action\":[{\"type\":\"command\",\"command\":\"togglePlay\",\"asset_model\":\"item.hints[0].asset\"},{\"type\":\"command\",\"command\":\"toggleShow\",\"asset\":\"hint\"}],\"type\":\"click\"},\"asset\":\"icon_hint\",\"x\":5,\"y\":35},\"id\":\"mcq_template_1\"},{\"image\":[{\"event\":{\"action\":{\"type\":\"command\",\"command\":\"show\",\"asset\":\"retryDialog\"},\"type\":\"click\"},\"asset\":\"popupTint\",\"x\":-100,\"y\":-150,\"w\":550,\"h\":600,\"visible\":true,\"id\":\"popup-Tint\"},{\"asset\":\"retryBg\",\"x\":0,\"y\":0,\"w\":150,\"h\":150,\"visible\":true,\"id\":\"right\"}],\"shape\":[{\"event\":{\"action\":[{\"type\":\"command\",\"command\":\"hide\",\"asset\":\"retryDialog\"},{\"type\":\"command\",\"command\":\"SHOWHTMLELEMENTS\",\"asset\":\"retry\"}],\"type\":\"click\"},\"type\":\"roundrect\",\"x\":72,\"y\":25,\"w\":50,\"h\":65,\"visible\":true,\"id\":\"retry\",\"hitArea\":true},{\"event\":{\"action\":{\"type\":\"command\",\"command\":\"transitionTo\",\"asset\":\"theme\",\"param\":\"next\",\"effect\":\"fadein\",\"direction\":\"left\",\"ease\":\"linear\",\"duration\":100},\"type\":\"click\"},\"type\":\"roundrect\",\"x\":110,\"y\":100,\"w\":25,\"h\":35,\"visible\":true,\"id\":\"continue\",\"hitArea\":true}],\"id\":\"retry\"},{\"g\":{\"image\":[{\"asset\":\"popupTint\",\"x\":0,\"y\":0,\"w\":100,\"h\":100,\"visible\":true,\"id\":\"popup-Tint\"}],\"text\":[{\"x\":25,\"y\":25,\"w\":50,\"h\":9,\"visible\":true,\"editable\":true,\"model\":\"word.lemma\",\"weight\":\"normal\",\"font\":\"helvetica\",\"color\":\"rgb(0,0,0)\",\"fontstyle\":\"\",\"fontsize\":75,\"align\":\"left\",\"z-index\":1,\"id\":\"lemma\"},{\"x\":25,\"y\":35,\"w\":50,\"h\":40,\"visible\":true,\"editable\":true,\"model\":\"word.gloss\",\"weight\":\"normal\",\"font\":\"helvetica\",\"color\":\"rgb(0,0,0)\",\"fontstyle\":\"\",\"fontsize\":43,\"align\":\"left\",\"z-index\":2,\"id\":\"gloss\"}],\"shape\":[{\"x\":20,\"y\":20,\"w\":60,\"h\":60,\"visible\":true,\"editable\":true,\"type\":\"roundrect\",\"radius\":10,\"opacity\":1,\"fill\":\"#45b3a5\",\"stroke-width\":1,\"z-index\":0,\"id\":\"textBg\"}],\"x\":0,\"y\":0,\"w\":100,\"h\":100,\"event\":{\"action\":[{\"type\":\"command\",\"command\":\"SHOWHTMLELEMENTS\",\"asset\":\"textBg\"},{\"type\":\"command\",\"command\":\"hide\",\"parent\":true}],\"type\":\"click\"}},\"id\":\"infoTemplate\"},{\"image\":[{\"event\":{\"action\":{\"type\":\"command\",\"command\":\"show\",\"asset\":\"\"},\"type\":\"click\"},\"asset\":\"popupTint\",\"x\":-100,\"y\":-150,\"w\":550,\"h\":600,\"visible\":true,\"id\":\"popup-Tint\"},{\"event\":{\"action\":[{\"type\":\"command\",\"command\":\"transitionTo\",\"asset\":\"theme\",\"param\":\"next\",\"effect\":\"fadein\",\"direction\":\"left\",\"ease\":\"linear\",\"duration\":500}],\"type\":\"click\"},\"asset\":\"goodjobBg\",\"x\":0,\"y\":0,\"w\":150,\"h\":150,\"visible\":true,\"id\":\"continue\"}],\"id\":\"goodjob\"}],\"stage\":[{\"id\":\"recordAudio7\",\"x\":0,\"y\":0,\"w\":100,\"h\":100,\"param\":[{\"name\":\"previousScreen\",\"value\":\"recordAudio6\"},{\"name\":\"nextScreen\",\"value\":\"recordAudio8\"},{\"name\":\"next\",\"value\":\"scene7b2e5492-b134-4945-9cde-803a094fb430\"},{\"name\":\"previous\",\"value\":\"recordAudio6\"}],\"events\":{\"event\":[{\"action\":[{\"asset\":\"startrec\",\"command\":\"hide\",\"type\":\"command\"},{\"asset\":\"stoprec\",\"command\":\"show\",\"type\":\"command\"},{\"asset\":\"mover\",\"command\":\"show\",\"type\":\"command\"},{\"tween\":{\"to\":[{\"duration\":0,\"ease\":\"linear\",\"__cdata\":{\"x\":47,\"y\":45}},{\"duration\":1000,\"ease\":\"sineInOut\",\"__cdata\":{\"x\":62,\"y\":45}},{\"duration\":1000,\"ease\":\"sineInOut\",\"__cdata\":{\"x\":47,\"y\":45}}],\"id\":\"mover\",\"loop\":true},\"asset\":\"mover\",\"type\":\"animation\"}],\"type\":\"rec_started\"},{\"action\":[{\"asset\":\"stoprec\",\"command\":\"hide\",\"type\":\"command\"},{\"asset\":\"mover\",\"command\":\"hide\",\"type\":\"command\"},{\"asset\":\"playback\",\"command\":\"show\",\"type\":\"command\"}],\"type\":\"rec_stopped\"},{\"action\":{\"type\":\"command\",\"command\":\"play\",\"asset\":\"domain_4002\",\"loop\":1},\"type\":\"enter\"},{\"action\":{\"type\":\"command\",\"command\":\"stop\",\"asset\":\"domain_4002\",\"loop\":1},\"type\":\"exit\"}]},\"image\":[{\"x\":16.38888888888889,\"y\":20.22222222222222,\"w\":69.44444444444444,\"h\":77.77777777777779,\"visible\":true,\"editable\":true,\"asset\":\"domain_3126\",\"z-index\":0},{\"x\":45,\"y\":80,\"w\":10,\"h\":18,\"visible\":true,\"editable\":true,\"asset\":\"mic\",\"event\":{\"action\":{\"asset\":\"recorder\",\"command\":\"toggleShow\",\"type\":\"command\"},\"type\":\"click\"},\"z-index\":6},{\"event\":{\"action\":{\"type\":\"command\",\"command\":\"transitionTo\",\"asset\":\"theme\",\"param\":\"next\",\"effect\":\"fadein\",\"direction\":\"left\",\"ease\":\"linear\",\"duration\":500},\"type\":\"click\"},\"asset\":\"next\",\"x\":93,\"y\":3,\"w\":5,\"h\":8.3,\"id\":\"next\",\"visible\":true,\"editable\":true,\"z-index\":100},{\"event\":{\"action\":{\"type\":\"command\",\"command\":\"transitionTo\",\"asset\":\"theme\",\"param\":\"previous\",\"effect\":\"fadein\",\"direction\":\"right\",\"ease\":\"linear\",\"duration\":100},\"type\":\"click\"},\"asset\":\"previous\",\"x\":2,\"y\":3,\"w\":5,\"h\":8.3,\"id\":\"previous\",\"visible\":true,\"editable\":true,\"z-index\":100}],\"text\":[{\"x\":25.27777777777778,\"y\":6.222222222222222,\"w\":71.11111111111111,\"h\":9.322222222222223,\"visible\":true,\"editable\":true,\"__text\":\"बिल्लियाँ देखती ही रह गई |\",\"weight\":\"normal\",\"font\":\"Helvetica\",\"color\":\"rgb(0,0,0)\",\"fontstyle\":\"\",\"fontsize\":85,\"lineHeight\":1.3,\"align\":\"left\",\"z-index\":7}],\"shape\":[],\"hotspot\":[{\"x\":25.208333333333332,\"y\":4.888888888888889,\"w\":14.722222222222223,\"h\":8.444444444444445,\"visible\":true,\"editable\":true,\"type\":\"roundrect\",\"radius\":1,\"fill\":\"red\",\"stroke-width\":1,\"keyword\":\"\",\"event\":[{\"action\":{\"type\":\"command\",\"command\":\"play\",\"asset\":\"domain_7637\"},\"type\":\"click\"}],\"hitArea\":true,\"z-index\":1},{\"x\":40.97222222222222,\"y\":5.111111111111112,\"w\":10.277777777777777,\"h\":8,\"visible\":true,\"editable\":true,\"type\":\"roundrect\",\"radius\":1,\"fill\":\"red\",\"stroke-width\":1,\"keyword\":\"\",\"event\":[{\"action\":{\"type\":\"command\",\"command\":\"play\",\"asset\":\"domain_7638\"},\"type\":\"click\"}],\"hitArea\":true,\"z-index\":2},{\"x\":52.361111111111114,\"y\":5.111111111111112,\"w\":3.75,\"h\":8.666666666666668,\"visible\":true,\"editable\":true,\"type\":\"roundrect\",\"radius\":1,\"fill\":\"red\",\"stroke-width\":1,\"keyword\":\"\",\"event\":[{\"action\":{\"type\":\"command\",\"command\":\"play\",\"asset\":\"domain_7639\"},\"type\":\"click\"}],\"hitArea\":true,\"z-index\":3},{\"x\":57.36111111111111,\"y\":7.333333333333333,\"w\":4.861111111111112,\"h\":6.888888888888889,\"visible\":true,\"editable\":true,\"type\":\"roundrect\",\"radius\":1,\"fill\":\"red\",\"stroke-width\":1,\"keyword\":\"\",\"event\":[{\"action\":{\"type\":\"command\",\"command\":\"play\",\"asset\":\"domain_7640\"},\"type\":\"click\"}],\"hitArea\":true,\"z-index\":4},{\"x\":63.47222222222222,\"y\":4.888888888888889,\"w\":5.555555555555555,\"h\":9.777777777777779,\"visible\":true,\"editable\":true,\"type\":\"roundrect\",\"radius\":1,\"fill\":\"red\",\"stroke-width\":1,\"keyword\":\"\",\"event\":[{\"action\":{\"type\":\"command\",\"command\":\"play\",\"asset\":\"domain_7641\"},\"type\":\"click\"}],\"hitArea\":true,\"z-index\":5}],\"embed\":[],\"div\":[],\"audio\":[{\"asset\":\"domain_4002\"},{\"asset\":\"domain_7637\"},{\"asset\":\"domain_7638\"},{\"asset\":\"domain_7639\"},{\"asset\":\"domain_7640\"},{\"asset\":\"domain_7641\"}],\"scribble\":[],\"g\":[{\"g\":[{\"image\":{\"event\":{\"action\":{\"asset\":\"recorder\",\"command\":\"hide\",\"type\":\"command\"},\"type\":\"click\"},\"asset\":\"blacktint\",\"h\":100,\"w\":100,\"x\":0,\"y\":0},\"h\":100,\"w\":100,\"x\":0,\"y\":0},{\"shape\":[{\"fill\":\"gray\",\"h\":20,\"type\":\"roundrect\",\"w\":30,\"x\":36,\"y\":37},{\"event\":{\"action\":{\"asset\":\"startrec\",\"command\":\"startRecord\",\"failure\":\"rec_start_fail\",\"success\":\"rec_started\",\"type\":\"command\"},\"type\":\"click\"},\"fill\":\"#34e941\",\"h\":20,\"type\":\"roundrect\",\"w\":30,\"x\":35,\"y\":35}],\"image\":{\"asset\":\"record\",\"h\":10,\"w\":6,\"x\":38,\"y\":40},\"text\":{\"color\":\"darkgreen\",\"font\":\"sans-serif\",\"fontsize\":50,\"h\":5,\"w\":20,\"x\":47,\"y\":43,\"__text\":\"Start\"},\"h\":100,\"id\":\"startrec\",\"w\":100,\"x\":0,\"y\":0},{\"shape\":[{\"fill\":\"gray\",\"h\":20,\"type\":\"roundrect\",\"w\":30,\"x\":36,\"y\":37},{\"event\":{\"action\":{\"asset\":\"stoprec\",\"command\":\"stopRecord\",\"failure\":\"rec_stop_failed\",\"success\":\"rec_stopped\",\"type\":\"command\"},\"type\":\"click\"},\"fill\":\"#e93441\",\"h\":20,\"type\":\"roundrect\",\"w\":30,\"x\":35,\"y\":35},{\"fill\":\"#e98888\",\"h\":2,\"id\":\"mover\",\"type\":\"circle\",\"visible\":false,\"w\":2,\"x\":47,\"y\":45}],\"image\":{\"asset\":\"stop_record\",\"h\":10,\"w\":6,\"x\":38,\"y\":40},\"h\":100,\"id\":\"stoprec\",\"visible\":false,\"w\":100,\"x\":0,\"y\":0},{\"shape\":[{\"fill\":\"gray\",\"h\":20,\"type\":\"roundrect\",\"w\":30,\"x\":36,\"y\":37},{\"event\":{\"action\":[{\"asset\":\"current_rec\",\"command\":\"TOGGLEPLAY\",\"type\":\"command\"},{\"asset\":\"recorder\",\"command\":\"hide\",\"type\":\"command\"},{\"asset\":\"playback\",\"command\":\"hide\",\"type\":\"command\"},{\"asset\":\"startrec\",\"command\":\"show\",\"type\":\"command\"}],\"type\":\"click\"},\"fill\":\"#34e941\",\"h\":20,\"type\":\"roundrect\",\"w\":30,\"x\":35,\"y\":35}],\"image\":{\"asset\":\"icon_sound\",\"h\":10,\"id\":\"story_audio_button\",\"w\":6,\"x\":38,\"y\":40},\"text\":{\"color\":\"darkgreen\",\"font\":\"sans-serif\",\"fontsize\":50,\"h\":5,\"w\":20,\"x\":47,\"y\":43,\"__text\":\"Play\"},\"h\":100,\"id\":\"playback\",\"visible\":false,\"w\":100,\"x\":0,\"y\":0}],\"h\":100,\"id\":\"recorder\",\"z-index\":200,\"visible\":false,\"w\":100,\"x\":0,\"y\":0}],\"appEvents\":{\"list\":\"rec_started,rec_start_fail,rec_stopped,rec_stop_failed\"}},{\"id\":\"scene185d692a-1841-4a35-96c1-67cebcde42a2\",\"x\":0,\"y\":0,\"w\":100,\"h\":100,\"param\":[{\"name\":\"next\",\"value\":\"scenea3cc8a52-833a-4a9d-b6e6-3be6b266b70f\"},{\"name\":\"previous\",\"value\":\"scene5de719ed-207c-4f01-a7e8-e1eb4abf77ed\"}],\"events\":{\"event\":[{\"action\":{\"type\":\"command\",\"command\":\"play\",\"asset\":\"domain_11825\",\"loop\":1},\"type\":\"enter\"},{\"action\":{\"type\":\"command\",\"command\":\"stop\",\"asset\":\"domain_11825\",\"loop\":1},\"type\":\"exit\"},{\"action\":[{\"type\":\"command\",\"command\":\"play\",\"asset\":\"retry_audio\"},{\"type\":\"command\",\"command\":\"show\",\"asset\":\"retryDialog\"},{\"tween\":{\"to\":{\"ease\":\"bounceOut \",\"duration\":1500,\"__cdata\":{\"x\":20,\"y\":20,\"w\":40,\"h\":30}},\"id\":\"retryShowAnim\"},\"type\":\"animation\",\"asset\":\"retryDialog\"}],\"type\":\"wrong_response\"},{\"action\":[{\"type\":\"command\",\"command\":\"play\",\"asset\":\"goodjob_audio\"},{\"type\":\"command\",\"command\":\"show\",\"asset\":\"goodjobDialog\"},{\"tween\":{\"to\":{\"ease\":\"bounceOut\",\"duration\":1500,\"__cdata\":{\"x\":20,\"y\":20,\"w\":40,\"h\":30}},\"id\":\"gjShowAnim\"},\"type\":\"animation\",\"asset\":\"goodjobDialog\"}],\"type\":\"correct_response\"}]},\"image\":[{\"event\":{\"action\":{\"type\":\"command\",\"command\":\"transitionTo\",\"asset\":\"theme\",\"param\":\"next\",\"effect\":\"fadein\",\"direction\":\"left\",\"ease\":\"linear\",\"duration\":500},\"type\":\"click\"},\"asset\":\"next\",\"x\":93,\"y\":3,\"w\":5,\"h\":8.3,\"id\":\"next\",\"visible\":true,\"editable\":true,\"z-index\":100},{\"event\":{\"action\":{\"type\":\"command\",\"command\":\"transitionTo\",\"asset\":\"theme\",\"param\":\"previous\",\"effect\":\"fadein\",\"direction\":\"right\",\"ease\":\"linear\",\"duration\":100},\"type\":\"click\"},\"asset\":\"previous\",\"x\":2,\"y\":3,\"w\":5,\"h\":8.3,\"id\":\"previous\",\"visible\":true,\"editable\":true,\"z-index\":100},{\"event\":{\"action\":[{\"type\":\"command\",\"command\":\"eval\",\"asset\":\"scene185d692a-1841-4a35-96c1-67cebcde42a2\",\"success\":\"correct_response\",\"failure\":\"wrong_response\"},{\"type\":\"command\",\"command\":\"HIDEHTMLELEMENTS\",\"asset\":\"scene185d692a-1841-4a35-96c1-67cebcde42a2\"}],\"type\":\"click\"},\"asset\":\"validate\",\"x\":91,\"y\":15,\"h\":15}],\"text\":[],\"shape\":[],\"hotspot\":[],\"embed\":[],\"div\":[],\"audio\":[{\"asset\":\"domain_11825\"}],\"scribble\":[],\"g\":[{\"embed\":{\"template\":\"item\",\"var-item\":\"item\"},\"x\":10,\"y\":0,\"w\":80,\"h\":90},{\"embed\":{\"template-name\":\"retry\"},\"x\":30,\"y\":-40,\"w\":40,\"h\":48,\"id\":\"retryDialog\",\"visible\":false},{\"embed\":{\"template-name\":\"goodjob\"},\"x\":30,\"y\":-40,\"w\":40,\"h\":48,\"id\":\"goodjobDialog\",\"visible\":false}],\"iterate\":\"assessment_185d692a-1841-4a35-96c1-67cebcde42a2\",\"preload\":true,\"var\":\"item\",\"appEvents\":{\"list\":\"next_item,correct_response,wrong_response\"}},{\"id\":\"scenea3cc8a52-833a-4a9d-b6e6-3be6b266b70f\",\"x\":0,\"y\":0,\"w\":100,\"h\":100,\"param\":[{\"name\":\"next\",\"value\":\"endScreen\"},{\"name\":\"previous\",\"value\":\"scene185d692a-1841-4a35-96c1-67cebcde42a2\"}],\"events\":{\"event\":[{\"action\":{\"type\":\"command\",\"command\":\"play\",\"asset\":\"domain_8313\",\"loop\":1},\"type\":\"enter\"},{\"action\":{\"type\":\"command\",\"command\":\"stop\",\"asset\":\"domain_8313\",\"loop\":1},\"type\":\"exit\"},{\"action\":[{\"type\":\"command\",\"command\":\"play\",\"asset\":\"retry_audio\"},{\"type\":\"command\",\"command\":\"show\",\"asset\":\"retryDialog\"},{\"tween\":{\"to\":{\"ease\":\"bounceOut \",\"duration\":1500,\"__cdata\":{\"x\":20,\"y\":20,\"w\":40,\"h\":30}},\"id\":\"retryShowAnim\"},\"type\":\"animation\",\"asset\":\"retryDialog\"}],\"type\":\"wrong_response\"},{\"action\":[{\"type\":\"command\",\"command\":\"play\",\"asset\":\"goodjob_audio\"},{\"type\":\"command\",\"command\":\"show\",\"asset\":\"goodjobDialog\"},{\"tween\":{\"to\":{\"ease\":\"bounceOut\",\"duration\":1500,\"__cdata\":{\"x\":20,\"y\":20,\"w\":40,\"h\":30}},\"id\":\"gjShowAnim\"},\"type\":\"animation\",\"asset\":\"goodjobDialog\"}],\"type\":\"correct_response\"}]},\"image\":[{\"event\":{\"action\":{\"type\":\"command\",\"command\":\"transitionTo\",\"asset\":\"theme\",\"param\":\"next\",\"effect\":\"fadein\",\"direction\":\"left\",\"ease\":\"linear\",\"duration\":500},\"type\":\"click\"},\"asset\":\"next\",\"x\":93,\"y\":3,\"w\":5,\"h\":8.3,\"id\":\"next\",\"visible\":true,\"editable\":true,\"z-index\":100},{\"event\":{\"action\":{\"type\":\"command\",\"command\":\"transitionTo\",\"asset\":\"theme\",\"param\":\"previous\",\"effect\":\"fadein\",\"direction\":\"right\",\"ease\":\"linear\",\"duration\":100},\"type\":\"click\"},\"asset\":\"previous\",\"x\":2,\"y\":3,\"w\":5,\"h\":8.3,\"id\":\"previous\",\"visible\":true,\"editable\":true,\"z-index\":100},{\"event\":{\"action\":[{\"type\":\"command\",\"command\":\"eval\",\"asset\":\"scenea3cc8a52-833a-4a9d-b6e6-3be6b266b70f\",\"success\":\"correct_response\",\"failure\":\"wrong_response\"},{\"type\":\"command\",\"command\":\"HIDEHTMLELEMENTS\",\"asset\":\"scenea3cc8a52-833a-4a9d-b6e6-3be6b266b70f\"}],\"type\":\"click\"},\"asset\":\"validate\",\"x\":91,\"y\":15,\"h\":15}],\"text\":[],\"shape\":[],\"hotspot\":[],\"embed\":[],\"div\":[],\"audio\":[{\"asset\":\"domain_8313\"}],\"scribble\":[],\"g\":[{\"embed\":{\"template\":\"item\",\"var-item\":\"item\"},\"x\":10,\"y\":0,\"w\":80,\"h\":90},{\"embed\":{\"template-name\":\"retry\"},\"x\":30,\"y\":-40,\"w\":40,\"h\":48,\"id\":\"retryDialog\",\"visible\":false},{\"embed\":{\"template-name\":\"goodjob\"},\"x\":30,\"y\":-40,\"w\":40,\"h\":48,\"id\":\"goodjobDialog\",\"visible\":false}],\"iterate\":\"assessment_a3cc8a52-833a-4a9d-b6e6-3be6b266b70f\",\"preload\":true,\"var\":\"item\",\"appEvents\":{\"list\":\"next_item,correct_response,wrong_response\"}},{\"id\":\"endScreen\",\"x\":0,\"y\":0,\"w\":100,\"h\":100,\"param\":[{\"name\":\"previousScreen\",\"value\":\"recordAudio9\"},{\"name\":\"nextScreen\",\"value\":\"splash\"},{\"name\":\"previous\",\"value\":\"scenea3cc8a52-833a-4a9d-b6e6-3be6b266b70f\"}],\"events\":{\"event\":[]},\"image\":[{\"x\":0,\"y\":0,\"w\":100,\"h\":100,\"visible\":true,\"editable\":true,\"asset\":\"endScreen\",\"z-index\":0},{\"x\":40,\"y\":50,\"w\":8,\"h\":13,\"visible\":false,\"editable\":true,\"asset\":\"previous\",\"event\":{\"action\":{\"asset\":\"theme\",\"command\":\"transitionTo\",\"direction\":\"right\",\"duration\":600,\"ease\":\"linear\",\"effect\":\"scroll\",\"param\":\"previousScreen\",\"type\":\"command\"},\"type\":\"click\"},\"z-index\":1},{\"x\":50,\"y\":50,\"w\":8,\"h\":13,\"visible\":false,\"editable\":true,\"asset\":\"next\",\"event\":{\"action\":{\"asset\":\"theme\",\"command\":\"transitionTo\",\"direction\":\"right\",\"duration\":600,\"ease\":\"linear\",\"effect\":\"scroll\",\"param\":\"nextScreen\",\"type\":\"command\"},\"type\":\"click\"},\"z-index\":2},{\"event\":{\"action\":{\"type\":\"command\",\"command\":\"transitionTo\",\"asset\":\"theme\",\"param\":\"previous\",\"effect\":\"fadein\",\"direction\":\"right\",\"ease\":\"linear\",\"duration\":100},\"type\":\"click\"},\"asset\":\"previous\",\"x\":2,\"y\":3,\"w\":5,\"h\":8.3,\"id\":\"previous\",\"visible\":true,\"editable\":true,\"z-index\":100}],\"text\":[],\"shape\":[],\"hotspot\":[],\"embed\":[],\"div\":[],\"audio\":[],\"scribble\":[],\"g\":[]}]}}"))
    )
    val result: Map[String, AnyRef] = ExtractableMimeTypeHelper.processECMLBody(obj, jobConfig)(ec, cloudStorageUtil)
    result.contains("artifactUrl") shouldBe true
    result.get("artifactUrl") shouldNot be(null)
    result.contains("cloudStorageKey") shouldBe true
    result.get("cloudStorageKey") shouldNot be(null)
  }

}
